#!/bin/bash
# Pear Limited
#  2015-09-11: Added Help

printhelp() {
echo "
Usage: sh parted.sh [OPTION]   (chmod 777 parted.sh)

  pboot                      pear_init_load
  install_exfat              install_exfat_nofuse
  pdns                       pear_dns 
  gdns                       config_dns
  point                      gen_mount_point
  gen_new_point              gen_new_point
  part                       parted_all
  mount                      mount_all
  multidial                  multidial
  testallppp                 testallppp
  tv                         install_teamview
  rebuildservice             rebuildservice
  ppp0_dial                  ppp0_dial (best use pppoeconf)
  satatest                   sata_directory_check
  rm_docker                  remove_service_container
  install_docker             install_docker
  tencent_down_test          tencent_down_test
  mount_on_running           mount_on_running
  install_tencent_sv_servie  install_tencent_sv_servie
  docker_down_test           docker_down_test
  *                          printhelp
"
}

source /root/pear/pear_config.sh

PEAR_MACHINE_NAME=""
PART_SIZE=186
PART_NUMBER=10
DATA_DISK_TYPE="ext4"   # exfat or xfs

USER="pear"

BandWidth_PER_ACCOUNT=50
DIAL_ACCOUNT_NUMBER=100

REGION="default"

LOGDIR="/root/pear/log"
MULTI_LOG="${LOGDIR}/pear_multi.log"
if [ ! -d "${LOGDIR}" ]; then
  mkdir ${LOGDIR}
fi

WORKDIR=`pwd`

sysctl -w vm.swappiness=0  > /dev/null 2>&1
result=`sysctl -p 2>&1`

ulimit -HSn 131072 
sed  -i '/ulimit -HSn 131072/d' /root/.bashrc
echo "ulimit -HSn 131072" >> /root/.bashrc 
source /root/.bashrc 


swapresult=`cat /etc/fstab | grep  '^#/swap'`
if [[ -z ${swapresult} ]]; then
  swaprecord=`cat /etc/fstab | grep  '^/swap'`
  if [[ -n ${swaprecord} ]]; then
    sed -ri 's/.*swap.*/#&/' /etc/fstab
  fi
fi
swapoff -a && swapon -a  

# cat /proc/vmstat | egrep "dirty|writeback" 

maxnumber=0
DISK_SIZES=` lsblk  | grep disk | grep -v loop | grep T | awk '{print $4}'  | uniq`
if [[ -z ${DISK_SIZES} ]]; then
  echo "no greater 1T data disk "
else
  for disk_size  in ${DISK_SIZES[@]}
  do
    number=`lsblk  | grep disk | grep -v loop | grep ${disk_size} | wc -l`
    if [[ ${number} -gt ${maxnumber} ]]; then
      maxnumber=${number}
      data_disk_size=${disk_size}
    fi
  done 
fi
DATA_DISK_SIZE=${data_disk_size}

# blkDisks=`lsblk  | grep disk | grep -v loop | grep ${DATA_DISK_SIZE} | awk '{system("echo /dev/" $1)}'`
blkDisks=`lsblk  | grep disk | grep -v loop | grep T | awk '{system("echo /dev/" $1)}'`
blkDisks=(${blkDisks})
ALL_DATADISK_NUMBER=${#blkDisks[@]}

declare -a dataDisks 
touch ${LOGDIR}/fdisk_error
for part in ${blkDisks[@]}
do
  outputresult=`fdisk -l ${part} `
  if [[ -z ${outputresult} ]]; then
    echo "Disk "${part} "Read Erro, chekc file:  ${LOGDIR}/fdisk_error"
    errorresult=`fdisk -l ${part} 2>&1`
    echo ${part}  ": "   ${errorresult}  >>  ${LOGDIR}/fdisk_error 
    continue
  fi
  dataDisks[${#dataDisks[@]}]=${part}
done
DISK_NUMBER=${#dataDisks[@]}

ALLPARTSNUMBER=`expr  ${DISK_NUMBER} \* ${PART_NUMBER} `


rootfilesystemname=`df -hT | grep ext4 | grep -e '/$' | awk '{print $1}'`
echo "root system: " ${rootfilesystemname}

local_lang=`locale | grep "LANG=" | awk -F= '{print $2}'`  
if [[ -z ${local_lang} ]]; then
  local_lang=` locale | grep LC_ALL | awk -F= '{print $2}'`  
fi
echo "language: " ${local_lang}

MAX_SPEED_NETCARD=""
find_max_speed_netcard() {
  netcards=`ifconfig | grep RUNNING | grep -v ppp | grep -v veth  | grep MULTICAST | awk '{print $1}' | sed -e 's/://g' | awk -F. '{print $1}' | uniq`
  maxnetcardspeed=0
  for netcard in ${netcards[@]}
  do
    # echo "check network card : "${netcard}
    cmd="ethtool ${netcard} | grep Speed | awk '{print \$2}' | awk -FM '{print \$1}'"
    echo ${cmd}
    echo ${netcard}
    netcardspeed=`ethtool ${netcard} | grep Speed | awk '{print $2}' | awk -FM '{print $1}'`
    
    if [[ ${netcardspeed} >  ${maxnetcardspeed}  ]]; then
      # echo "netcardspeed: "   ${netcardspeed} 
      maxnetcardspeed=${netcardspeed}
      maxnetcard=${netcard}
    fi
  done
  # echo "maxnetcard " ${maxnetcard} "maxnetcardspeed: " ${maxnetcardspeed} 
  if [[ -z ${maxnetcard} ]]; then
     echo  "no available network card "
     return 
  fi
  MAX_SPEED_NETCARD=${maxnetcard}
}

find_max_speed_netcard

if [[ -z ${MAX_SPEED_NETCARD} ]]; then
  echo "no available network card "
  # exit 1
fi

declare -a AVAILABLE_PARTITIONS 
parted_all() {
  if [[ ${DATA_DISK_TYPE} == "exfat" ]]; then
    detectexfat=`lsmod | grep exfat`
    if [[ -z ${detectexfat} ]]; then
       install_exfat_nofuse
    fi 
  fi

  PART_LOG=${LOGDIR}/part

  if [ ! -d ${PART_LOG} ]; then
    mkdir  -p  ${PART_LOG}
  fi

  echo "PART_SIZE:" ${PART_SIZE} " PART_NUMBER:" ${PART_NUMBER} " Total PARTS: "  ${ALLPARTSNUMBER}
  sed -i '/pear_part/d' /root/pear/${PEAR_MACHINE_NAME}_parm.conf

  for part in ${dataDisks[@]}
  do
    echo "now check " ${part}
    length=${#part}
    if [[ ${rootfilesystemname:0:${length}} == ${part} ]]; then
      echo "skip root system " ${rootfilesystemname}
      continue
    fi

    echo "check current part is valid"
    echo "check current disk if being used"
    mountpoints=` df | grep ${part} | awk '{print $6}' `
    if [[ -n ${mountpoints} ]]; then
      for mountpoint in ${mountpoints[@]}; do
        cmd="umount ${mountpoint}"
        ${cmd} 2> temp
        if [[ -n ${temp} ]]; then
          echo ${temp}  
          break
        fi
      done      
    fi

    mountpoints=` df | grep ${part} | awk '{print $6}' `
    if [[ -n ${mountpoints} ]]; then
      echo ${part} "is busy, could not be umounted "
      echo "pear_part: " ${part} "is busy"  >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
      continue 
    fi

    if [[ ${PART_NUMBER} == "1" ]]; then
      cmd="parted --script ${part} mklabel gpt mkpart primary 100MiB ${PART_SIZE}GiB"
    elif [[  ${PART_NUMBER} == "5"  ]]; then
      cmd="parted --script ${part} mklabel gpt \
        mkpart primary  100MiB                ${PART_SIZE}GiB  \
        mkpart primary  ${PART_SIZE}GiB        $(( PART_SIZE*2 ))GiB \
        mkpart primary  $(( PART_SIZE*2 ))GiB $(( PART_SIZE*3 ))GiB \
        mkpart primary  $(( PART_SIZE*3 ))GiB $(( PART_SIZE*4 ))GiB \
        mkpart primary  $(( PART_SIZE*4 ))GiB $(( PART_SIZE*5 ))GiB"
    elif [[  ${PART_NUMBER} == "10"  ]]; then
      cmd="parted --script ${part} mklabel gpt \
        mkpart primary  100MiB                ${PART_SIZE}GiB  \
        mkpart primary  ${PART_SIZE}GiB        $(( PART_SIZE*2 ))GiB \
        mkpart primary  $(( PART_SIZE*2 ))GiB $(( PART_SIZE*3 ))GiB \
        mkpart primary  $(( PART_SIZE*3 ))GiB $(( PART_SIZE*4 ))GiB \
        mkpart primary  $(( PART_SIZE*4 ))GiB $(( PART_SIZE*5 ))GiB\
        mkpart primary  $(( PART_SIZE*5 ))GiB $(( PART_SIZE*6 ))GiB \
        mkpart primary  $(( PART_SIZE*6 ))GiB $(( PART_SIZE*7 ))GiB \
        mkpart primary  $(( PART_SIZE*7 ))GiB $(( PART_SIZE*8 ))GiB \
        mkpart primary  $(( PART_SIZE*8 ))GiB $(( PART_SIZE*9 ))GiB \
        mkpart primary  $(( PART_SIZE*9 ))GiB $(( PART_SIZE*10 ))GiB"
    else

       echo $part
       disk_size=`lsblk ${part} -b -o SIZE  | grep -m 1  -vi SIZE`
       echo disk size is  $disk_size
       disk_size=$(( ${disk_size} / 1000 / 1000 / 1000 ))
       max_parts=$(( ${disk_size} / ${PART_SIZE}  ))
       echo $disk_size $PART_NUMBER $PART_SIZE   $max_parts
       if expr ${PART_NUMBER} > ${max_parts}
       then
             echo "too many parts"
             PART_NUMBER=${max_parts}
       fi
       PART_NUMBER=${max_parts}
       echo $disk_size $PART_NUMBER  $PART_SIZE   $max_parts


       cmd="parted --script ${part} mklabel gpt "
       for k in `seq 0 $((PART_NUMBER-1))`
       do
           if expr $k = 0 
           then 
               cmd=${cmd}"mkpart primary  100MiB  ${PART_SIZE}GiB "
               continue
           fi 
           cmd=${cmd}"mkpart primary  $(( PART_SIZE*k))GiB $(( PART_SIZE*(k+1) ))GiB "
       done 
    fi

    echo ${cmd}
    # added by ym for support which partition to parted begin
    if [ "l${SELECT_PARTITON}" = "lTRUE" ] ;
    then
        echo "Do you want to parted ${part} (wait 15s), continue(y/n):"
        read -t 15 answer
        if [[ -n ${answer} ]]; then
          if [[ ${answer} = "n" ]]; then
            continue
          fi
        fi
    fi 
    echo "parted something"
    # added by ym for support which partition to parted end

    touch ${PART_LOG}/part_error.txt
    touch ${PART_LOG}/part_info.txt

    ${cmd} 2> ${PART_LOG}/part_error.txt  1> ${PART_LOG}/part_info.txt

    cmderror=`cat  ${PART_LOG}/part_error.txt`
    if [[ -n  ${cmderror} ]]; then
      echo ${cmderror}
      echo "pear_part: " ${part} "partition fail "  >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
      continue 
    fi

    sleep 5  

    for i in `seq 1 ${PART_NUMBER}`
    do 
      mydevice=${part}${i}
      if [[ $part =~ "nvme" ]]
      then
          echo "its nvme storage"
          mydevice=${part}p${i}
      fi
      if [[ ${DATA_DISK_TYPE} == "exfat" ]]; then
        cmd="mkfs.exfat    ${mydevice}"
      elif [[ ${DATA_DISK_TYPE} == "xfs" ]]; then
        cmd="mkfs.xfs  -f  -l lazy-count=1 ${mydevice}"
      elif [[ ${DATA_DISK_TYPE} == "ext4" ]]; then
        cmd="mkfs.ext4  -F -O ^has_journal -T largefile  ${mydevice} "
      fi

      echo ${cmd}
      touch ${PART_LOG}/format_error.txt
      touch ${PART_LOG}/format_info.txt

      ${cmd} 2>${PART_LOG}/format_error.txt   1>${PART_LOG}/format_info.txt
      cmderror=`cat ${PART_LOG}/format_error.txt`
      if [[ -n  ${cmderror} ]]; then
        echo ${cmderror}
        echo "pear_part: " ${part}${i} "format fail,  " ${cmderror}  >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
        continue 
      fi
      AVAILABLE_PARTITIONS[${#AVAILABLE_PARTITIONS[@]}]=${mydevice}
    done
  done
  touch /root/pear/available_partitions.result 
  echo  ${AVAILABLE_PARTITIONS[@]}  > /root/pear/available_partitions.result 
}

gen_new_point() {
  rm -rf /sata* 
  gen_mount_point
}

gen_mount_point() {
  for i in `seq 0 $((DIAL_ACCOUNT_NUMBER-1)) `
  do 
    cmd="mkdir -p /sata${i}"
    ${cmd}
  done
  chmod -R 777 /sata*
  sataNumber=`ls / | grep sata | wc -l`
  echo "create /sata* : " ${sataNumber}
}

declare -a WORK_SATAS
declare -a FAIL_SATAS

mount_all() {
  for i in `seq 0 $((DIAL_ACCOUNT_NUMBER-1)) `
  do
    cmd="umount /sata${i}"
    echo ${cmd}
    cmdresult=$(${cmd}  2>&1)
    while [[ -z ${cmdresult} ]]; do
      sleep 2
      cmdresult=$(${cmd}  2>&1)
    done
  done 
  source /root/pear/pear_mount_on_running.sh
}

mount_on_running() {
  echo "begin mount sata disk ......"

  if [[ ${REGION} == "jiangxi" ]]; then
      part="/dev/sda"
      index=0
      for i in `seq 1 16`
      do
        cmd="mount  ${part}${i} /sata${index}"
        echo ${cmd}
        ${cmd}
        let index=index+1
      done
      mount  /dev/nvme0n1p1 /sata16
      mount  /dev/nvme0n1p2 /sata17
      mount  /dev/nvme1n1p1 /sata18
      mount  /dev/nvme1n1p2 /sata19
  else
      echo "begin mount default sata disk "
      AVAILABLE_PARTITIONS=`cat /root/pear/available_partitions.result `
      AVAILABLE_PARTITIONS=(${AVAILABLE_PARTITIONS})
      echo "available Data Partitions : " ${AVAILABLE_PARTITIONS[@]}
      AVAILABLE_PARTITIONS_NUMBER=${#AVAILABLE_PARTITIONS[@]}
      echo "available Data Partitions Number: " ${AVAILABLE_PARTITIONS_NUMBER}

      index=0
      for part in ${AVAILABLE_PARTITIONS[@]}
      do
        echo "now mount : " ${part}  "to /sata"${index}
        length=${#part}
        if [[ ${rootfilesystemname:0:${length}} == ${part} ]]; then
          echo "skip root system: " ${rootfilesystemname}
          continue
        fi

        if [[ ${DATA_DISK_TYPE} == "exfat" ]]; then
          cmd="mount -o rw,noatime,nodiratime,lazytime ${part} /sata${index}  "
        elif [[  ${DATA_DISK_TYPE} == "xfs" ]]; then
          cmd="mount -o rw,noatime,nodiratime,lazytime,noiversion,seclabel  ${part} /sata${index}"
        elif [[ ${DATA_DISK_TYPE} == "ext4" ]]; then
          cmd="to do"
        fi

        echo ${cmd}
        ${cmd} 2> ${LOGDIR}/mountresult.txt
        temp=`cat ${LOGDIR}/mountresult.txt`
        if [[ -n  ${temp} ]]; then
          echo "mount /sata"${index}"  fail" 
          continue 
        fi

        if [[ ${DATA_DISK_TYPE} == "exfat" ]]; then
          exfatresult=`mount | grep 'sata'${index}' '  | awk '{print $5}'`
          if [[ ${exfatresult} != "exfat" ]]; then
            echo "not exfat, please check exfat module , for example lsmod | grep exfat "
            exit 
          fi
        fi

        ls  /sata${index} 2> "${PART_LOG}/sata"${index}"_error.txt"
        result=`cat   "${PART_LOG}/sata"${index}"_error.txt" `
        if [[ -z ${result} ]]; then
          WORK_SATAS[${#WORK_SATAS[@]}]=/sata${index}
        else
          FAIL_SATAS[${#FAIL_SATAS[@]}]=/sata${index}
        fi
        let index=index+1
      done 
      echo "available work satss: " ${WORK_SATAS[@]}  >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
      echo "available work satss number: " ${#WORK_SATAS[@]} >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
      echo "abnormal satas: " ${FAIL_SATAS[@]}  >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf 
      echo "abnormal satas number: " ${#FAIL_SATAS[@]} >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf   
  fi

}

install_exfat_nofuse() {

  apt remove -y exfat-fuse
  apt autoremove
  rmmod exfat

  apt install -y exfat-utils 
  WORKDIR=`pwd`
  if [[ ! -d  "./version" ]]; then
    mkdir version
  fi

  cd version
  versiondate=` date  +%Y%m%d_%H%M%S`
  mkdir ${versiondate}
  cd ${versiondate}
  wget http://ali.webrtc.win/pear_engine_service/exfat-nofuse.tar.gz
  filemd5=`md5sum  exfat-nofuse.tar.gz | awk '{print $1}'`

  current_exfat_md5=`cat ${WORKDIR}/version/current_exfat_md5.txt `

  if [[ ${filemd5} != ${current_exfat_md5}  ]]; then
    echo ${versiondate}  ":  "  ${filemd5} >> ${WORKDIR}/version/history_exfat_md5.txt 
    echo ${filemd5}  >  ${WORKDIR}/version/current_exfat_md5.txt
  fi


  # git clone https://github.com/gameblabla/exfat-nofuse
  tar -C ./ -zxvf exfat-nofuse.tar.gz
  cd exfat-nofuse
  make
  make install

  insmod exfat.ko
  echo  "====================================="  > ${WORKDIR}/version/${versiondate}/exfat.log
  modinfo exfat  >> ${WORKDIR}/version/${versiondate}/exfat.log
  modprobe -D exfat >> ${WORKDIR}/version/${versiondate}/exfat.log
  echo  "=====================================" >> ${WORKDIR}/version/${versiondate}/exfat.log

  cd ..
  rm -rf ./exfat-nofuse*

  # added for reboot 
  if grep exfat /etc/modules; then
     echo "exfat has added"
  else
     echo "add exfat"
     echo exfat >> /etc/modules
  fi
}
install_docker() {
  sudo apt  -y remove    docker docker-engine docker.io
  sudo apt  -y install 
  sudo apt  -y install   apt-transport-https 
  sudo apt  -y install   ca-certificates 
  sudo apt  -y install   curl 
  sudo apt  -y install   software-properties-common

  curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo apt-key add -
  sudo add-apt-repository  "deb [arch=amd64] https://download.docker.com/linux/ubuntu  $(lsb_release -cs)  stable"
  sudo apt   update
  sudo apt  -y install docker-ce
  sudo docker version 

  sudo groupadd docker
  sudo gpasswd -a ${USER} docker
  sudo service docker restart
  newgrp - docker
}

install_teamview() {
  mkdir teamview
  cd teamview
  wget https://download.teamviewer.com/download/linux/teamviewer_amd64.deb
  sudo apt-get update
  sudo dpkg -i teamviewer_amd64.deb
  sudo apt-get   --fix-broken install 
  sudo dpkg -i teamviewer_amd64.deb
}

config_dns(){
  sed -i '/DNS=/d'  /etc/systemd/resolved.conf  
  echo "DNS=119.29.29.29" >> /etc/systemd/resolved.conf

  sed -i '/FallbackDNS=/d'  /etc/systemd/resolved.conf  
  echo "FallbackDNS=180.76.76.76" >> /etc/systemd/resolved.conf

  sudo systemctl restart systemd-resolved.service
  sudo systemd-resolve --flush-caches
  # systemd-resolve --status 
  # systemctl status systemd-resolved.service
}

pear_dns(){
  
  if [[ ! -f /etc/init.d/pear_ddns ]]; then
    wget http://ali.webrtc.win/tool/pear_ddns.tar.gz 
    tar -C / -zxvf  pear_ddns.tar.gz   

    sed -i "s/pear_ddns \&/pear_ddns --sub_domain=${PEAR_MACHINE_NAME}  --nic=ppp0 \&/g" /etc/init.d/pear_ddns    
    /etc/init.d/pear_ddns start  
  fi
}

stop_pppd_monitor() {
  killall -9 pear_ppp_monitor.sh
  killall -9 pear_findppp
  sleep 3
}

stop_pppd() {
  
  # kill all virtual pppd 
  ps -aux | grep pppd | grep options |grep -viE 'color|monitor'| awk '{print $2}' | awk '{system("kill -9  " $1)}'

  pppdresult=`ps -aux | grep pppd | grep options |grep -viE 'color|monitor'| awk '{print $2}'`
  while [[ -n ${pppdresult} ]]; do
    sleep 1
    pppdresult=`ps -aux | grep pppd | grep options |grep -viE 'color|monitor'| awk '{print $2}'`
  done
}

pear_multiple_dial(){

  if [[ -z ${PEAR_MACHINE_NAME} ]]; then
    echo "please input PEAR_MACHINE_NAME, default is common: "
    read PEAR_MACHINE_NAME
    if [[ -z ${PEAR_MACHINE_NAME} ]]; then
      PEAR_MACHINE_NAME="common"
    fi
  fi

  cd /root/pear/multidial

  DIAL_ACCOUNT_NAME=${PEAR_MACHINE_NAME}"_account.txt"

  if [[ ! -f  ${DIAL_ACCOUNT_NAME}  ]]; then
    echo "no ${PEAR_MACHINE_NAME}_account.txt !!!!!!!!!!!! "
    exit 1
  fi

  ./pear_multiple -b -g ${DIAL_ACCOUNT_NAME} -w aa:bb:cc:dd:ee:ff
}


remove_service_container(){
  exiteddockers=`docker ps -a | grep tencent | grep  Exited |  awk '{print $14}' | awk '{system("docker  rm  -f   " $1)}'  `

  checkresult=`docker ps -a | grep tencent `
  while [[  -n  ${checkresult} ]]; do
    sleep 3
    checkresult=`docker ps -a | grep tencent `
  done
}

restart_business()  {

  if [[ -f /etc/init.d/xc_cdn.sh ]]; then
    /etc/init.d/xc_cdn.sh restart
  fi
}

rebuildservice()  {

  cd /root/pear/multidial
  echo `date +'%Y-%m-%d %H:%M:%S'` rebuildservice  >>  ${MULTI_LOG}

  if [[ -f /etc/init.d/xc_cdn.sh ]]; then
    /etc/init.d/xc_cdn.sh stop
  fi

    tencentdockers=`docker ps  | grep tencent  | awk '{print $12}' `
  for tencentdokcer in ${tencentdockers[@]}
  do
    docker stop ${tencentdokcer}
  done     

    multidial

  docker ps -a | grep tencent | grep  Exited |  awk '{print $14}' | awk '{system("docker restart  " $1)}'

  if [[ -f /etc/init.d/xc_cdn.sh ]]; then
    /etc/init.d/xc_cdn.sh start 
  fi
}

ppp0_dial() {
  ppp0_inet_status=`ifconfig ppp0 | grep inet`
  if [[ -z ${ppp0_inet_status} ]]; then
    cd  /etc/ppp/peers/ 
    pon dsl-provider
    main_dial_ts=`date +%s`
    touch ${LOGDIR}/ppp0_${main_dial_ts}_result
    plog >> ${LOGDIR}/ppp0_${main_dial_ts}_result
    sleep 2
    plog >> ${LOGDIR}/ppp0_${main_dial_ts}_result
    sleep 2
    plog >> ${LOGDIR}/ppp0_${main_dial_ts}_result
  fi

  ppp0_inet_status=`ifconfig ppp0 | grep inet`
  if [[ -n ${ppp0_inet_status} ]]; then
    echo "ppp0 normal exit  "   >> ${LOGDIR}/ppp0_${main_dial_ts}_result 
  else
    echo "ppp0 fail"   >> ${LOGDIR}/ppp0_${main_dial_ts}_result 
  fi

}

down_virtural_card() {
  virtual_net_cars=`ifconfig | grep veth | awk '{print $1}' | sed -e 's/://g' `
  if [[ -n ${virtual_net_cars} ]]; then
    for virtual_net_car in ${virtual_net_cars[@]}
    do
      ifconfig  ${virtual_net_car}  down 
      cmd="ip link delete ${virtual_net_car}"
      echo ${cmd}
      ${cmd}

      ifconfig ${virtual_net_car} 2> ${LOGDIR}/${virtual_net_car}.status 
      temp=`cat ${LOGDIR}/${virtual_net_car}.status `
      echo ${temp}

      while [[ -z ${temp} ]]; do
        sleep 2
        ifconfig ${virtual_net_car} 2> ${LOGDIR}/${virtual_net_car}.status 
        temp=`cat ${LOGDIR}/${virtual_net_car}.status `
        echo ${temp}
      done
    done
  else
    virtual_net_cars=`ip link | grep veth |  awk '{print $2}' | sed -e 's/://g'  | awk -F@ '{print $1}'`
    if [[ -n ${virtual_net_cars} ]]; then
      for virtual_net_car in ${virtual_net_cars[@]}
      do
        cmd="ip link delete ${virtual_net_car}"
        echo ${cmd}
        ${cmd}        
      done
    fi
  fi
}

multidial() {

  cd /root/pear/multidial
  echo "------------------------------------------------------------------------" >>  ${MULTI_LOG}    
  echo `date +'%Y-%m-%d %H:%M:%S'` begin multidial  >>  ${MULTI_LOG}
  check_ppp0_status
  echo `date +'%Y-%m-%d %H:%M:%S'` check_ppp0_status  >>  ${MULTI_LOG}

  stop_pppd_monitor
  echo `date +'%Y-%m-%d %H:%M:%S'` stop_pppd_monitor  >>  ${MULTI_LOG}

  stop_pppd 
  echo `date +'%Y-%m-%d %H:%M:%S'` stop_pppd  >>  ${MULTI_LOG}

  down_virtural_card
  echo `date +'%Y-%m-%d %H:%M:%S'` down_virtural_card  >>  ${MULTI_LOG}

  # added by ym for multi fibre begin
  if test -f /root/pear/veth.sh ;
  then 
    echo "has veth.sh"
    source /root/pear/veth.sh
  else
    echo "no veth.sh"
    add_virtural_link
  fi
  # added by ym for multi fibre end
  
  echo `date +'%Y-%m-%d %H:%M:%S'` add_virtural_link  >>  ${MULTI_LOG}

  pear_multiple_dial
  echo `date +'%Y-%m-%d %H:%M:%S'` pear_multiple_dial  >>  ${MULTI_LOG}

  create_multiple_dial_route
  echo `date +'%Y-%m-%d %H:%M:%S'` create_multiple_dial_route  >>  ${MULTI_LOG}


  start_ppp_monitor
  echo `date +'%Y-%m-%d %H:%M:%S'` start_ppp_monitor >>  ${MULTI_LOG}
  testallppp
  echo `date +'%Y-%m-%d %H:%M:%S'` testallppp >>  ${MULTI_LOG}
}

add_virtural_link() {

  WORKING_NETCARD=${MAX_SPEED_NETCARD}
  echo "INITIAL WORKING_NETCARD: "  ${WORKING_NETCARD}

  VLANCARD=`cat /proc/net/vlan/config | grep ${WORKING_NETCARD} | awk -F\| '{print $1}' | sed 's/ //g' `
  echo "VLANCARD: "${VLANCARD}
  
  if [[ -n ${VLANCARD} ]]; then
    result=`ifconfig  ${VLANCARD}  | grep RUNNING`
    echo "result: " ${result}

    if [[ -n ${result} ]]; then
      WORKING_NETCARD=${VLANCARD}
    else
      echo "vlan card not running, please check it "
      exit 0
    fi
  fi
  
  echo "WORKING_NETCARD: " ${WORKING_NETCARD}

  for i in `seq 0 $((DIAL_ACCOUNT_NUMBER-1)) `
  do 
    cmd="sudo ip link add link ${WORKING_NETCARD} name veth${i} type macvlan "
    echo ${cmd}
    ${cmd}
  done

}

check_ppp0_status() {
  # get the first point to point interface 
  ppp_interface=`ip -o link | grep POINTOPOINT | grep -iv 'state DOWN'| awk '{print $2}' | sed -e 's/://g' | sort -g | head -1`

  if [[ -z ${ppp_interface} ]]; then
    echo "no point to point interface up"
    exit 1
  fi

  if  ! ping -I ${ppp_interface} -c 4 -w 30 qq.com > /dev/null ;
  then
      echo " ${ppp_interface}  fail "
      exit 1
  fi
}

start_ppp_monitor() {
  cd /root/pear/multidial
  nohup /root/pear/multidial/pear_ppp_monitor.sh &   
}

create_multiple_dial_route() {
  nohup /root/pear/multidial/pear_findppp & 
}


testallppp() {
  echo "LOGDIR: " ${LOGDIR}
  rm -f ${LOGDIR}/ppp_*.result 

  ppptimestamp=`date +%s`
  touch ${LOGDIR}/ppp_${ppptimestamp}.result 

  if [[  ${REGION} == "jiangxi" ]]; then
    PPP_NUMBER=20
  else
    PPP_NUMBER=${DIAL_ACCOUNT_NUMBER}
  fi

  for i in `seq 0 $((PPP_NUMBER-1)) `  
  do 
    cmd="ping -c 2 -w 3 -I ppp${i} qq.com "
    echo ${cmd}
    pingresult=$(${cmd}  2>&1)
    echo ${pingresult}
    echo ppp${i} " : " ${pingresult}  >>  ${LOGDIR}/ppp_${ppptimestamp}.result 
  done

  cmd="cat ${LOGDIR}/ppp_${ppptimestamp}.result | grep -v ' 2 received' | awk '{print \$1}'"
  echo ${cmd}
  failresult=`cat ${LOGDIR}/ppp_${ppptimestamp}.result | grep -v ' 2 received' | awk '{print $1}'   2>&1 `


  sed -i '/ppptest/d'  /root/pear/${PEAR_MACHINE_NAME}_parm.conf
  for fail_ppp_netcard in ${failresult[@]}
  do
    fail_id=${fail_ppp_netcard:3}
    if [[ ${fail_id} == 0 ]]; then
      echo "ppp0 error, stop test, please handle ppp0 firstly"
      break  
    fi

    unit=`cat /etc/ppp/options.prfile$((fail_id-1)) | grep unit | awk '{print $2}'`
    if [[ ${unit} != ${fail_id} ]]; then
      break  
    fi
    fail_user=`cat /etc/ppp/options.prfile$((fail_id-1)) | grep user  | awk '{print $2}' | awk -F\' '{print $2}'`
    fail_vethnetcard=`cat /etc/ppp/options.prfile$((fail_id-1))  | grep veth | awk '{print $3}'`

    cmd="ps -aux | grep pppd | grep 'prfile$((fail_id-1)) ' |  awk '{print $2}'"
    fail_ps_id=`ps -aux | grep pppd | grep prfile$((fail_id-1))$ |  awk '{print $2}'` 
    echo "ppptest: " ${ppptimestamp} "_" ${fail_ppp_netcard} "_" ${fail_user}  "_"   ${fail_vethnetcard}  "_"  ${fail_ps_id} >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf
  done 
}

sata_directory_check() {
  sata_detect_ts=`date +%s`
  touch ${LOGDIR}/sata${sata_detect_ts}
  for i in `seq 0 $((ALLPARTSNUMBER-1)) `
  do 
    cd /sata${i}
    result=` echo "ok" >> checkdiskresult`
    if [[ -n ${result} ]]; then
      echo "/sata"${i} "  error: " ${result} >> ${LOGDIR}/sata${sata_detect_ts}
    fi
  done
}

single_dial() {
  echo "single_dial"
}

install_tencent_sv_servie(){

  echo "install tencent short video service ...... "	

  cmd="pear_monitor -d --auto_disk=/sata,0,$((DIAL_ACCOUNT_NUMBER-1)) --auto_nic=ppp,0,$((DIAL_ACCOUNT_NUMBER-1)) --auto_mac=${DIAL_ACCOUNT_NUMBER}"
  echo ${cmd}
  ${cmd} 2> temp
  macaddr=`cat temp | grep select | awk '{print $NF}'`
  interfacename=` cat temp  | grep select | awk '{print $((NF-1))}' | sed -e 's/,//g'`
  echo "mac used by oss2: "  ${interfacename} " : " ${macaddr} >> /root/pear/${PEAR_MACHINE_NAME}_parm.conf

  rm -f temp

  echo "finished install tencent short video service ! "

}

source  /root/pear/tools/business_test.sh

pear_init_load() {
  name="pear.service"
  she="pear_init"
  cp -r /root/gitee/auto_install/pear_init/etc/* /etc
  chmod 777  /etc/init.d/pear_init 
  chmod 777  /etc/systemd/system/pear.service 
  sudo systemctl daemon-reload
  sudo systemctl enable ${name}
}

remount() {
  for i in `seq 0 $((DIAL_ACCOUNT_NUMBER-1))`; 
  do 
    if [[ ${DATA_DISK_TYPE} == "exfat" ]]; then
      cmd="mount -o rw,remount,noatime,nodiratime,lazytime /sata${index}  "
    elif [[  ${DATA_DISK_TYPE} == "xfs" ]]; then
      cmd="mount -o rw,remount,noatime,nodiratime,lazytime,noiversion,attr2,inode64 /sata${index} "
    elif [[ ${DATA_DISK_TYPE} == "ext4" ]]; then
      cmd="mount -o rw,remount,noatime,nodiratime,lazytime,noiversion,nobarrier,noquota /sata${index}  "
    fi
  done
}

case "$1" in
  remount)
    remount
    ;;
  pboot)
    pear_init_load
    ;;
  install_exfat)
    install_exfat_nofuse
    ;;
  pdns)
    pear_dns 
    ;;
  gdns)
    config_dns
    ;;  
  point)
    gen_mount_point
    ;;
  gen_new_point)
    gen_new_point
    ;;
  part)
    parted_all
    ;;
  mount)
    mount_all
    ;;
  multidial)
    multidial
    ;;
  testallppp)
    testallppp
    ;;
  tv)
    install_teamview
    ;;
  rebuildservice)
    rebuildservice
    ;;
  ppp0_dial)
    ppp0_dial
    ;;
  satatest)
    sata_directory_check
    ;;
  rm_docker)
    remove_service_container
    ;;
  install_docker)
    install_docker
    ;;
  tencent_down_test)
    tencent_down_test
    ;;
  mount_on_running)
    mount_on_running
    ;;
  install_tencent_sv_servie)
    install_tencent_sv_servie
    ;;
  docker_down_test)
    docker_down_test
    ;;
  *)
    printhelp
    exit 0
esac
